import type { DisplayData, DisplayJSONData, DisplayNodeData, IMAlgorithmKey } from "./data_types";
import * as d3 from 'd3';

export function parseCoarseningData(obj: DisplayJSONData): DisplayData {
  // With preprocessing
  const res: DisplayData = {
    ...obj,
    graphs: obj.graphs.map((G) => {
      const nodeMap = new Map<string, DisplayNodeData>()
      G.nodes.forEach((node) => {
        nodeMap.set(node.id, node);
      });
      return { nodes: nodeMap, links: G.links };
    }),
    seeds: {
      cwim: obj.seeds.cwim.map((seeds) => new Set(seeds)),
      contrast: new Set(obj.seeds.contrast),
    },
  };
  return res;
}

function nSeeds(seeds: Array<string> | Set<string>) {
  if (seeds instanceof Array) {
    return seeds.length;
  } else if (seeds instanceof Set) {
    return seeds.size;
  } else {
    throw TypeError("Invalid seeds data type.");
  }
}

export function nTotalVisitedNodes(data: DisplayJSONData | DisplayData, key: IMAlgorithmKey) {
  const simData = data.simulation[key];
  const nVisited = d3.sum(simData.map((level) => level.length));
  if (key === 'cwim') {
    const seeds = data.seeds.cwim[0];
    return nVisited + nSeeds(seeds);
  } else if (key === 'contrast') {
    const seeds = data.seeds.contrast;
    return nVisited + nSeeds(seeds);
  } else {
    throw RangeError(`Invalid algorithm key '${key}'.`);
  }
}
